@@ -1,15 +1,12 @@
 package Mouse;
 use 5.006_002;
 
-use strict;
-use warnings;
+use Mouse::Exporter; # enables strict and warnings
 
-our $VERSION = '0.37_02';
+our $VERSION = '0.37_03';
 
-use Exporter;
-
-use Carp 'confess';
-use Scalar::Util 'blessed';
+use Carp qw(confess);
+use Scalar::Util qw(blessed);
 
 use Mouse::Util qw(load_class is_class_loaded get_code_package not_supported);
 
@@ -20,22 +17,29 @@ use Mouse::Meta::Attribute;
 use Mouse::Object;
 use Mouse::Util::TypeConstraints ();
 
-our @ISA = qw(Exporter);
+Mouse::Exporter->setup_import_methods(
+    as_is => [qw(
+        extends with
+        has
+        before after around
+        override super
+        augment  inner
+    ),
+        \&Scalar::Util::blessed,
+        \&Carp::confess,
+   ],
+);
 
+# XXX: for backward compatibility
 our @EXPORT = qw(
     extends with
     has
     before after around
     override super
     augment  inner
-
     blessed confess
 );
 
-our %is_removable = map{ $_ => undef } @EXPORT;
-delete $is_removable{blessed};
-delete $is_removable{confess};
-
 sub extends { Mouse::Meta::Class->initialize(scalar caller)->superclasses(@_) }
 
 sub has {
@@ -126,9 +130,6 @@ sub init_meta {
     my $base_class = $args{base_class} || 'Mouse::Object';
     my $metaclass  = $args{metaclass}  || 'Mouse::Meta::Class';
 
-    confess("The Metaclass $metaclass must be a subclass of Mouse::Meta::Class.")
-            unless $metaclass->isa('Mouse::Meta::Class');
-
     # make a subtype for each Mouse class
     Mouse::Util::TypeConstraints::class_type($class)
         unless Mouse::Util::TypeConstraints::find_type_constraint($class);
@@ -145,62 +146,6 @@ sub init_meta {
     return $meta;
 }
 
-sub import {
-    my $class = shift;
-
-    strict->import;
-    warnings->import;
-
-    my $opts = do {
-        if (ref($_[0]) && ref($_[0]) eq 'HASH') {
-            shift @_;
-        } else {
-            +{ };
-        }
-    };
-    my $level = delete $opts->{into_level};
-       $level = 0 unless defined $level;
-    my $caller = caller($level);
-
-    # we should never export to main
-    if ($caller eq 'main') {
-        warn qq{$class does not export its sugar to the 'main' package.\n};
-        return;
-    }
-
-    $class->init_meta(
-        for_class  => $caller,
-    );
-
-    if (@_) {
-        __PACKAGE__->export_to_level( $level+1, $class, @_);
-    } else {
-        # shortcut for the common case of no type character
-        no strict 'refs';
-        for my $keyword (@EXPORT) {
-            *{ $caller . '::' . $keyword } = *{__PACKAGE__ . '::' . $keyword};
-        }
-    }
-}
-
-sub unimport {
-    my $caller = caller;
-
-    my $stash = do{
-        no strict 'refs';
-        \%{$caller . '::'}
-    };
-
-    for my $keyword (@EXPORT) {
-        my $code;
-        if(exists $is_removable{$keyword}
-            && ($code = $caller->can($keyword))
-            && get_code_package($code) eq __PACKAGE__){
-
-            delete $stash->{$keyword};
-        }
-    }
-}
 
 1;
 
