@@ -1,35 +1,36 @@
 package Mouse::Meta::Method::Constructor;
-use strict;
-use warnings;
+use Mouse::Util; # enables strict and warnings
 
-sub _generate_constructor_method {
+sub _generate_constructor {
     my ($class, $metaclass, $args) = @_;
 
     my $associated_metaclass_name = $metaclass->name;
+
     my @attrs         = $metaclass->get_all_attributes;
 
     my $buildall      = $class->_generate_BUILDALL($metaclass);
     my $buildargs     = $class->_generate_BUILDARGS($metaclass);
     my $processattrs  = $class->_generate_processattrs($metaclass, \@attrs);
 
-    my @compiled_constraints = map { $_ ? $_->_compiled_type_constraint : undef }
-                               map { $_->type_constraint } @attrs;
-
-
+    my @checks = map { $_ && $_->_compiled_type_constraint }
+                 map { $_->type_constraint } @attrs;
 
     my $source = sprintf("#line %d %s\n", __LINE__, __FILE__).<<"...";
         sub \{
             my \$class = shift;
             return \$class->Mouse::Object::new(\@_)
                 if \$class ne q{$associated_metaclass_name};
+            # BUILDARGS
             $buildargs;
             my \$instance = bless {}, \$class;
+            # process attributes
             $processattrs;
+            # BUILDALL
             $buildall;
             return \$instance;
         }
 ...
-
+    #warn $source;
     my $code;
     my $e = do{
         local $@;
@@ -37,9 +38,7 @@ sub _generate_constructor_method {
         $@;
     };
     die $e if $e;
-
-    $metaclass->add_method($args->{constructor_name} => $code);
-    return;
+    return $code;
 }
 
 sub _generate_processattrs {
@@ -49,35 +48,50 @@ sub _generate_processattrs {
     my $has_triggers;
 
     for my $index (0 .. @$attrs - 1) {
-        my $attr = $attrs->[$index];
-        my $key  = $attr->name;
         my $code = '';
 
-        if (defined $attr->init_arg) {
-            my $from = $attr->init_arg;
+        my $attr = $attrs->[$index];
+        my $key  = $attr->name;
 
-            $code .= "if (exists \$args->{q{$from}}) {\n";
+        my $init_arg        = $attr->init_arg;
+        my $type_constraint = $attr->type_constraint;
+        my $need_coercion;
 
-            my $value = "\$args->{q{$from}}";
-            if(my $type_constraint = $attr->type_constraint){
-                if($attr->should_coerce && $type_constraint->has_coercion){
-                    $code .= "my \$value = \$attrs[$index]->{type_constraint}->coerce(\$args->{q{$from}});\n";
-                    $value = '$value';
-                }
+        my $instance_slot  = "\$instance->{q{$key}}";
+        my $attr_var       = "\$attrs[$index]";
+        my $constraint_var;
 
-                $code .= "\$compiled_constraints[$index]->($value)\n";
-                $code .= "  or \$attrs[$index]->verify_type_constraint_error(q{$key}, $value, \$attrs[$index]->{type_constraint});\n";
+        if(defined $type_constraint){
+             $constraint_var = "$attr_var\->{type_constraint}";
+             $need_coercion  = ($attr->should_coerce && $type_constraint->has_coercion);
+        }
+
+        $code .= "# initialize $key\n";
+
+        my $post_process = '';
+        if(defined $type_constraint){
+            $post_process .= "\$checks[$index]->($instance_slot)";
+            $post_process .= "  or $attr_var->verify_type_constraint_error(q{$key}, $instance_slot, $constraint_var);\n";
+        }
+        if($attr->is_weak_ref){
+            $post_process .= "Scalar::Util::weaken($instance_slot) if ref $instance_slot;\n";
+        }
+
+        if (defined $init_arg) {
+            my $value = "\$args->{q{$init_arg}}";
+
+            $code .= "if (exists $value) {\n";
+
+            if($need_coercion){
+                $value = "$instance_slot = $constraint_var->coerce($value);\n";
             }
 
-            $code .= "\$instance->{q{$key}} = $value;\n";
-
-            if ($attr->is_weak_ref) {
-                $code .= "Scalar::Util::weaken( \$instance->{q{$key}} ) if ref($value);\n";
-            }
+            $code .= "$instance_slot = $value;\n";
+            $code .= $post_process;
 
             if ($attr->has_trigger) {
                 $has_triggers++;
-                $code .= "push \@triggers, [\$attrs[$index]->{trigger}, $value];\n";
+                $code .= "push \@triggers, [$attr_var\->{trigger}, $value];\n";
             }
 
             $code .= "\n} else {\n";
@@ -88,55 +102,32 @@ sub _generate_processattrs {
                 my $default = $attr->default;
                 my $builder = $attr->builder;
 
-                $code .= "my \$value = ";
-
-                if ($attr->should_coerce && $attr->type_constraint) {
-                    $code .= "\$attrs[$index]->_coerce_and_verify(";
-                }
-
-                if ($attr->has_builder) {
-                    $code .= "\$instance->$builder()";
+                my $value;
+                if (defined($builder)) {
+                    $value = "\$instance->$builder()";
                 }
                 elsif (ref($default) eq 'CODE') {
-                    $code .= "\$attrs[$index]->{default}->(\$instance)";
+                    $value = "$attr_var\->{default}->(\$instance)";
                 }
-                elsif (!defined($default)) {
-                    $code .= 'undef';
-                }
-                elsif ($default =~ /^\-?[0-9]+(?:\.[0-9]+)$/) {
-                    $code .= $default;
+                elsif (defined($default)) {
+                    $value = "$attr_var\->{default}";
                 }
                 else {
-                    $code .= "'$default'";
+                    $value = 'undef';
                 }
 
-                if ($attr->should_coerce) {
-                    $code .= ");\n";
-                }
-                else {
-                    $code .= ";\n";
+                if($need_coercion){
+                    $value = "$constraint_var->coerce($value)";
                 }
 
-                if ($attr->has_type_constraint) {
-                    $code .= "{
-                        unless (\$attrs[$index]->{type_constraint}->check(\$value)) {
-                            \$attrs[$index]->verify_type_constraint_error(q{$key}, \$value, \$attrs[$index]->type_constraint)
-                        }
-                    }";
-                }
-
-                $code .= "\$instance->{q{$key}} = \$value;\n";
-
-                if ($attr->is_weak_ref) {
-                    $code .= "Scalar::Util::weaken( \$instance->{q{$key}} ) if ref( \$value );\n";
-                }
+                $code .= "$instance_slot = $value;\n";
             }
         }
         elsif ($attr->is_required) {
             $code .= "Carp::confess('Attribute ($key) is required');";
         }
 
-        $code .= "}\n" if defined $attr->init_arg;
+        $code .= "}\n" if defined $init_arg;
 
         push @res, $code;
     }
@@ -154,9 +145,10 @@ sub _generate_processattrs {
 }
 
 sub _generate_BUILDARGS {
-    my($self, $metaclass) = @_;
+    my(undef, $metaclass) = @_;
 
-    if ($metaclass->name->can('BUILDARGS') && $metaclass->name->can('BUILDARGS') != \&Mouse::Object::BUILDARGS) {
+    my $class = $metaclass->name;
+    if ( $class->can('BUILDARGS') && $class->can('BUILDARGS') != \&Mouse::Object::BUILDARGS ) {
         return 'my $args = $class->BUILDARGS(@_)';
     }
 
@@ -174,7 +166,7 @@ sub _generate_BUILDARGS {
 }
 
 sub _generate_BUILDALL {
-    my ($class, $metaclass) = @_;
+    my (undef, $metaclass) = @_;
 
     return '' unless $metaclass->name->can('BUILD');
 
